"""Top-level package for aa-contacts."""

__author__ = """Matteo Ghia"""
__email__ = 'matteo.ghia@yahoo.it'
__version__ = '0.9.0'

__github_url__ = 'https://github.com/Maestro-Zacht/aa-contacts'
__app_name_ua__ = 'aa-contacts'



from typing import ClassVar

from collections import defaultdict

from django.db import models
from django.utils import timezone
from django.contrib.auth.models import User

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveAllianceInfo, EveCharacter, EveCorporationInfo, EveFactionInfo
from esi.models import Token


class General(models.Model):
    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ('manage_alliance_contacts', 'Can manage alliance contacts'),
            ('manage_corporation_contacts', 'Can manage corporation contacts'),
            ('view_alliance_notes', 'Can view notes on alliance contacts'),
            ('view_corporation_notes', 'Can view notes on corporation contacts'),
        )


class ContactTokenQueryset(models.QuerySet):
    def with_valid_tokens(self):
        valid_tokens = Token.objects.all().require_valid()
        return self.filter(token__in=valid_tokens)


class ContactTokenManager(models.Manager):
    def get_queryset(self):
        return ContactTokenQueryset(self.model, using=self._db)

    def with_valid_tokens(self):
        return self.get_queryset().with_valid_tokens()


class ContactQueryset(models.QuerySet):
    def about(self, user_filter, only_main):
        base_query = CharacterOwnership.objects.filter(user=user_filter)
        if only_main:
            base_query = base_query.filter(character=models.F('user__profile__main_character'))

        user_characters = base_query.values('character__character_id')

        user_alliances = (
            base_query
            .filter(character__alliance_id__isnull=False)
            .values('character__alliance_id')
        )

        user_corps = base_query.values('character__corporation_id')

        user_factions = (
            base_query
            .filter(character__faction_id__isnull=False)
            .values('character__faction_id')
        )

        return self.filter(
            models.Q(contact_id__in=user_characters, contact_type=Contact.ContactTypeOptions.CHARACTER) |
            models.Q(contact_id__in=user_corps, contact_type=Contact.ContactTypeOptions.CORPORATION) |
            models.Q(contact_id__in=user_alliances, contact_type=Contact.ContactTypeOptions.ALLIANCE) |
            models.Q(contact_id__in=user_factions, contact_type=Contact.ContactTypeOptions.FACTION)
        )

    def with_contact_name(self):
        return self.annotate(
            contact_name_annotation=models.Case(
                models.When(
                    contact_type=Contact.ContactTypeOptions.CHARACTER,
                    then=models.Subquery(
                        EveCharacter.objects
                        .filter(character_id=models.OuterRef('contact_id'))
                        .values('character_name')
                    )
                ),
                models.When(
                    contact_type=Contact.ContactTypeOptions.CORPORATION,
                    then=models.Subquery(
                        EveCorporationInfo.objects
                        .filter(corporation_id=models.OuterRef('contact_id'))
                        .values('corporation_name')
                    )
                ),
                models.When(
                    contact_type=Contact.ContactTypeOptions.ALLIANCE,
                    then=models.Subquery(
                        EveAllianceInfo.objects
                        .filter(alliance_id=models.OuterRef('contact_id'))
                        .values('alliance_name')
                    )
                ),
                models.When(
                    contact_type=Contact.ContactTypeOptions.FACTION,
                    then=models.Subquery(
                        EveFactionInfo.objects
                        .filter(faction_id=models.OuterRef('contact_id'))
                        .values('faction_name')
                    )
                ),
                default=models.Value(''),
            )
        )


class ContactManager(models.Manager):
    def get_queryset(self):
        return ContactQueryset(self.model, using=self._db)

    def about(self, user_filter, only_main):
        return self.get_queryset().about(user_filter, only_main)

    def with_contact_name(self):
        return self.get_queryset().with_contact_name()


class ContactLabel(models.Model):
    label_id = models.BigIntegerField()
    label_name = models.CharField(max_length=255)

    class Meta:
        abstract = True
        default_permissions = ()


class Contact(models.Model):
    contact_id = models.BigIntegerField()

    class ContactTypeOptions(models.TextChoices):
        CHARACTER = 'character'
        CORPORATION = 'corporation'
        ALLIANCE = 'alliance'
        FACTION = 'faction'

    contact_type = models.CharField(max_length=11, choices=ContactTypeOptions.choices)
    standing = models.FloatField()
    notes = models.TextField(blank=True, default='')

    objects: ClassVar[ContactManager] = ContactManager()

    class Meta:
        abstract = True
        default_permissions = ()

    @property
    def image_src(self) -> str:
        if self.contact_type == self.ContactTypeOptions.CHARACTER:
            return EveCharacter.generic_portrait_url(self.contact_id)
        if self.contact_type == self.ContactTypeOptions.CORPORATION:
            return EveCorporationInfo.generic_logo_url(self.contact_id)
        if self.contact_type == self.ContactTypeOptions.ALLIANCE:
            return EveAllianceInfo.generic_logo_url(self.contact_id)
        if self.contact_type == self.ContactTypeOptions.FACTION:
            return EveFactionInfo.generic_logo_url(self.contact_id)
        return ''

    @property
    def _load_contact_name(self) -> str:
        if self.contact_type == self.ContactTypeOptions.CHARACTER:
            try:
                res = EveCharacter.objects.get(character_id=self.contact_id).character_name
            except EveCharacter.DoesNotExist:
                char = EveCharacter.objects.create_character(self.contact_id)
                res = char.character_name
        elif self.contact_type == self.ContactTypeOptions.CORPORATION:
            try:
                res = EveCorporationInfo.objects.get(corporation_id=self.contact_id).corporation_name
            except EveCorporationInfo.DoesNotExist:
                corp = EveCorporationInfo.objects.create_corporation(self.contact_id)
                res = corp.corporation_name
        elif self.contact_type == self.ContactTypeOptions.ALLIANCE:
            try:
                res = EveAllianceInfo.objects.get(alliance_id=self.contact_id).alliance_name
            except EveAllianceInfo.DoesNotExist:
                alliance = EveAllianceInfo.objects.create_alliance(self.contact_id)
                res = alliance.alliance_name
        elif self.contact_type == self.ContactTypeOptions.FACTION:
            try:
                res = EveFactionInfo.objects.get(faction_id=self.contact_id).faction_name
            except EveFactionInfo.DoesNotExist:
                faction = EveFactionInfo.provider.get_faction(self.contact_id)
                EveFactionInfo.objects.create(faction_id=faction.id, faction_name=faction.name)
                res = faction.name
        else:
            raise ValueError(f"Unknown contact type: {self.contact_type}")

        return res

    @property
    def contact_name(self) -> str:
        if (
            not hasattr(self, 'contact_name_annotation')
            or self.contact_name_annotation is None
            or self.contact_name_annotation == ''
        ):
            return self._load_contact_name
        else:
            return self.contact_name_annotation

    @classmethod
    def filter_missing_contact_name(cls, pk_list: list[int]) -> list[int]:
        return list(
            cls.objects
            .filter(pk__in=pk_list)
            .annotate(
                has_object=models.Case(
                    models.When(
                        contact_type=cls.ContactTypeOptions.CHARACTER,
                        then=models.Exists(EveCharacter.objects.filter(character_id=models.OuterRef('contact_id')))
                    ),
                    models.When(
                        contact_type=cls.ContactTypeOptions.CORPORATION,
                        then=models.Exists(EveCorporationInfo.objects.filter(corporation_id=models.OuterRef('contact_id')))
                    ),
                    models.When(
                        contact_type=cls.ContactTypeOptions.ALLIANCE,
                        then=models.Exists(EveAllianceInfo.objects.filter(alliance_id=models.OuterRef('contact_id')))
                    ),
                    models.When(
                        contact_type=cls.ContactTypeOptions.FACTION,
                        then=models.Exists(EveFactionInfo.objects.filter(faction_id=models.OuterRef('contact_id')))
                    ),
                    default=models.Value(False),
                    output_field=models.BooleanField()
                )
            )
            .filter(has_object=False)
            .values_list('pk', flat=True)
        )

    @classmethod
    def can_view_notes(cls, user: User) -> bool:
        raise NotImplementedError("Please implement in subclass")

    @classmethod
    def can_edit_notes(cls, user: User) -> bool:
        raise NotImplementedError("Please implement in subclass")


class ContactToken(models.Model):
    token = models.ForeignKey(Token, on_delete=models.CASCADE, related_name='+')

    last_update = models.DateTimeField(default=timezone.now)

    objects: ClassVar[ContactTokenManager] = ContactTokenManager()

    class Meta:
        abstract = True
        default_permissions = ()


class AllianceContactLabel(ContactLabel):
    alliance = models.ForeignKey(EveAllianceInfo, on_delete=models.RESTRICT, related_name='contact_labels')

    class Meta:
        default_permissions = ()

    def __str__(self):
        return f"{self.alliance} - {self.label_name}"


class AllianceContact(Contact):
    alliance = models.ForeignKey(EveAllianceInfo, on_delete=models.RESTRICT, related_name='contacts')

    labels = models.ManyToManyField(AllianceContactLabel, blank=True, related_name='contacts')

    class Meta:
        default_permissions = ()

    def __str__(self):
        return f"{self.alliance} - {self.contact_name}"

    @classmethod
    def can_view_notes(cls, user: User) -> bool:
        return user.has_perm('aa_contacts.view_alliance_notes')

    @classmethod
    def can_edit_notes(cls, user: User) -> bool:
        return user.has_perm('aa_contacts.manage_alliance_contacts') and cls.can_view_notes(user)


class AllianceToken(ContactToken):
    alliance = models.OneToOneField(EveAllianceInfo, on_delete=models.RESTRICT, related_name='+')

    class Meta:
        default_permissions = ()

    def __str__(self) -> str:
        return f"{self.alliance} Token"

    @classmethod
    def visible_for(cls, user):
        if user.is_superuser:
            return cls.objects.all()

        return cls.objects.filter(
            alliance__alliance_id__in=CharacterOwnership.objects
            .filter(user=user)
            .values('character__alliance_id')
        )


class CorporationContactLabel(ContactLabel):
    corporation = models.ForeignKey(EveCorporationInfo, on_delete=models.RESTRICT, related_name='contact_labels')

    class Meta:
        default_permissions = ()

    def __str__(self):
        return f"{self.corporation} - {self.label_name}"


class CorporationContact(Contact):
    corporation = models.ForeignKey(EveCorporationInfo, on_delete=models.RESTRICT, related_name='contacts')

    labels = models.ManyToManyField(CorporationContactLabel, blank=True, related_name='contacts')

    is_watched = models.BooleanField(null=True, blank=True, default=None)

    class Meta:
        default_permissions = ()

    def __str__(self):
        return f"{self.corporation} - {self.contact_name}"

    @classmethod
    def can_view_notes(cls, user: User) -> bool:
        return user.has_perm('aa_contacts.view_corporation_notes')

    @classmethod
    def can_edit_notes(cls, user: User) -> bool:
        return user.has_perm('aa_contacts.manage_corporation_contacts') and cls.can_view_notes(user)


class CorporationToken(ContactToken):
    corporation = models.OneToOneField(EveCorporationInfo, on_delete=models.RESTRICT, related_name='+')

    class Meta:
        default_permissions = ()

    def __str__(self) -> str:
        return f"{self.corporation} Token"

    @classmethod
    def visible_for(cls, user):
        if user.is_superuser:
            return cls.objects.all()

        return cls.objects.filter(
            corporation__corporation_id__in=CharacterOwnership.objects
            .filter(user=user)
            .values('character__corporation_id')
        )

# Secure Groups integration


class BaseFilter(models.Model):
    name = models.CharField(max_length=500)  # This is the filters name shown to the admin
    description = models.CharField(max_length=500)  # this is what is shown to the user

    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.name}: {self.description}"

    def process_filter(self, user: User):  # Single User Pass Fail system
        raise NotImplementedError("Please Create a filter!")

    def audit_filter(self, users):  # Bulk check system that also advises the user with simple messages
        raise NotImplementedError("Please Create an audit function!")


class StandingFilter(BaseFilter):
    class ComparisonOptions(models.TextChoices):
        GREATER_THAN = '>'
        GREATER_OR_EQUAL = '>='
        LESS_THAN = '<'
        LESS_OR_EQUAL = '<='
        EQUAL = '='

    comparison = models.CharField(max_length=2, choices=ComparisonOptions.choices)
    standing = models.FloatField()

    class CheckTypeOptions(models.TextChoices):
        AT_LEAST_ONE_CHARACTER = 'any'
        ALL_CHARACTERS = 'all'
        NO_CHARACTER = 'no'

    check_type = models.CharField(max_length=3, choices=CheckTypeOptions.choices, default=CheckTypeOptions.AT_LEAST_ONE_CHARACTER)
    only_main = models.BooleanField(default=False, help_text="Only consider the main character of the user")

    corporations = models.ManyToManyField(EveCorporationInfo, blank=True, related_name='corp_standing_filters', help_text="The corporations that have set the standings")
    alliances = models.ManyToManyField(EveAllianceInfo, blank=True, related_name='alliance_standing_filters', help_text="The alliances that have set the standings")

    class Meta:
        verbose_name = "Smart Filter: User Standings"
        verbose_name_plural = verbose_name
        default_permissions = ()

    @property
    def _standing_lookup(self):
        if self.comparison == self.ComparisonOptions.GREATER_THAN:
            return models.Q(standing__gt=self.standing)
        elif self.comparison == self.ComparisonOptions.GREATER_OR_EQUAL:
            return models.Q(standing__gte=self.standing)
        elif self.comparison == self.ComparisonOptions.LESS_THAN:
            return models.Q(standing__lt=self.standing)
        elif self.comparison == self.ComparisonOptions.LESS_OR_EQUAL:
            return models.Q(standing__lte=self.standing)
        else:
            return models.Q(standing=self.standing)

    def _corp_query(self, user_filter):
        return (
            CorporationContact.objects
            .about(user_filter, self.only_main)
            .filter(
                self._standing_lookup,
                corporation__in=self.corporations.all()
            )
        )

    def _alliance_query(self, user_filter):
        return (
            AllianceContact.objects
            .about(user_filter, self.only_main)
            .filter(
                self._standing_lookup,
                alliance__in=self.alliances.all()
            )
        )

    def process_filter(self, user: User) -> bool:
        return self.audit_filter(User.objects.filter(pk=user.pk))[user.pk]['check']

    def audit_filter(self, users):
        output = defaultdict(lambda: {"message": "", "check": False})

        if self.check_type == StandingFilter.CheckTypeOptions.AT_LEAST_ONE_CHARACTER:
            annotated_query = users.annotate(
                check=models.Exists(
                    self._corp_query(models.OuterRef(models.OuterRef('pk')))
                ) | models.Exists(
                    self._alliance_query(models.OuterRef(models.OuterRef('pk')))
                )
            )

            for user in annotated_query.values('pk', 'check'):
                output[user['pk']] = {
                    "message": "User has a character, corporation, alliance or faction that meets the filter" if user['check'] else "User does not meet the filter",
                    "check": user['check']
                }
        elif self.check_type == StandingFilter.CheckTypeOptions.NO_CHARACTER:
            annotated_query = users.annotate(
                check=~(
                    models.Exists(
                        self._corp_query(models.OuterRef(models.OuterRef('pk')))
                    ) | models.Exists(
                        self._alliance_query(models.OuterRef(models.OuterRef('pk')))
                    )
                )
            )

            for user in annotated_query.values('pk', 'check'):
                output[user['pk']] = {
                    "message": "User has all characters that meet the filter" if user['check'] else "User does not meet the filter",
                    "check": user['check']
                }
        else:
            base_query = CharacterOwnership.objects.filter(user=models.OuterRef('pk'))
            if self.only_main:
                base_query = base_query.filter(character=models.F('user__profile__main_character'))

            chars = (
                base_query.filter(
                    models.Exists(
                        CorporationContact.objects.filter(
                            self._standing_lookup,
                            models.Q(
                                contact_id=models.OuterRef('character__character_id'),
                                contact_type=Contact.ContactTypeOptions.CHARACTER
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__corporation_id'),
                                contact_type=Contact.ContactTypeOptions.CORPORATION
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__alliance_id'),
                                contact_type=Contact.ContactTypeOptions.ALLIANCE
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__faction_id'),
                                contact_type=Contact.ContactTypeOptions.FACTION
                            ),
                            corporation__in=self.corporations.all()
                        )
                    ) |
                    models.Exists(
                        AllianceContact.objects.filter(
                            self._standing_lookup,
                            models.Q(
                                contact_id=models.OuterRef('character__character_id'),
                                contact_type=Contact.ContactTypeOptions.CHARACTER
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__corporation_id'),
                                contact_type=Contact.ContactTypeOptions.CORPORATION
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__alliance_id'),
                                contact_type=Contact.ContactTypeOptions.ALLIANCE
                            ) |
                            models.Q(
                                contact_id=models.OuterRef('character__faction_id'),
                                contact_type=Contact.ContactTypeOptions.FACTION
                            ),
                            alliance__in=self.alliances.all()
                        )
                    )
                )
                .values('user')
                .annotate(count=models.Count('*'))
                .values('count')
            )

            annotated_query = (
                users
                .annotate(count_valid=models.Subquery(chars))
                .annotate(total=models.Subquery(
                    base_query
                    .values('user')
                    .annotate(count=models.Count('*'))
                    .values('count')
                ))
            )

            for user in annotated_query.values('pk', 'count_valid', 'total'):
                passed = user['count_valid'] == user['total']
                output[user['pk']] = {
                    "message": "User has all characters that meet the filter" if passed else "User does not meet the filter",
                    "check": passed
                }

        return output



from django.apps import AppConfig
from aa_contacts import __version__


class AAContactsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "aa_contacts"
    verbose_name = f"Contacts v{__version__}"



from ninja import Router

from django.contrib.auth.models import User

from allianceauth.authentication.models import CharacterOwnership
from aa_contacts.models import AllianceToken

from ..schema import AllianceTokenSchema


router = Router()


@router.get("/", response=list[AllianceTokenSchema])
def get_list(request):
    return AllianceToken.visible_for(request.user)


@router.get("/{int:alliance_id}/", response={200: AllianceTokenSchema, 403: None, 404: None})
def get_single(request, alliance_id: int):
    user: User = request.user

    ownerships = CharacterOwnership.objects.filter(user=user)
    if not user.is_superuser and not ownerships.filter(character__alliance_id=alliance_id).exists():
        return 403, None

    try:
        token = AllianceToken.visible_for(user).get(alliance__alliance_id=alliance_id)
    except AllianceToken.DoesNotExist:
        return 404, None

    return 200, token



from ninja import NinjaAPI
from ninja.security import django_auth

from django.conf import settings

from .alliance import router as alliance_router
from .corporation import router as corporation_router
from .permissions import router as permissions_router

api = NinjaAPI(
    title="AA Contacts API",
    version="0.0.1",
    urls_namespace='aa_contacts:api',
    auth=django_auth,
    csrf=True,
    openapi_url=settings.DEBUG and "/openapi.json" or ""
)

api.add_router("/alliances", alliance_router)
api.add_router("/corporations", corporation_router)
api.add_router("/permissions", permissions_router)



from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib import messages
from django.utils.translation import gettext as _

from esi.decorators import token_required
from esi.models import Token
from allianceauth.eveonline.models import EveCharacter, EveAllianceInfo, EveCorporationInfo

from .models import AllianceContact, AllianceToken, CorporationToken, CorporationContact
from .tasks import update_alliance_contacts, update_corporation_contacts
from .forms import AllianceContactForm, CorporationContactForm
from . import __version__


@login_required
def index(request):
    return redirect('aa_contacts:react_view')


@login_required
def react_view(request):
    context = {
        'version': __version__,
    }

    return render(request, 'aa_contacts/react_base.html', context=context)


@login_required
def dashboard(request):
    context = {
        'alliance_tokens': AllianceToken.visible_for(request.user).select_related('alliance'),
        'corporation_tokens': CorporationToken.visible_for(request.user).select_related('corporation'),
    }
    return render(request, 'aa_contacts/index.html', context=context)


@login_required
def alliance_contacts(request, alliance_pk: int):
    try:
        token = AllianceToken.visible_for(request.user).select_related('alliance').get(alliance_id=alliance_pk)
    except AllianceToken.DoesNotExist:
        messages.error(request, _('You do not have the permissions for viewing this alliance contacts.'))
        return redirect('aa_contacts:dashboard')

    contacts = (
        AllianceContact.objects
        .with_contact_name()
        .filter(alliance=token.alliance)
        .prefetch_related('labels')
    )

    context = {
        'contacts': contacts,
        'token': token,
        'alliance': token.alliance,
    }

    return render(request, 'aa_contacts/alliance_contacts.html', context=context)


@login_required
def corporation_contacts(request, corporation_pk: int):
    try:
        token = CorporationToken.visible_for(request.user).select_related('corporation').get(corporation_id=corporation_pk)
    except CorporationToken.DoesNotExist:
        messages.error(request, _('You do not have the permissions for viewing this corporation contacts.'))
        return redirect('aa_contacts:dashboard')

    contacts = (
        CorporationContact.objects
        .with_contact_name()
        .filter(corporation=token.corporation)
        .prefetch_related('labels')
    )

    context = {
        'contacts': contacts,
        'token': token,
        'corporation': token.corporation,
    }

    return render(request, 'aa_contacts/corporation_contacts.html', context=context)


@login_required
@permission_required('aa_contacts.manage_alliance_contacts')
@token_required(scopes=['esi-alliances.read_contacts.v1'])
def add_alliance_token(request, token: Token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)

    if char.alliance_id is None:
        messages.error(request, _('You need to be in an alliance to add alliance contacts.'))
        return redirect('aa_contacts:dashboard')

    try:
        alliance = char.alliance
    except EveAllianceInfo.DoesNotExist:
        alliance = EveAllianceInfo.objects.create_alliance(char.alliance_id)

    if AllianceToken.objects.filter(alliance=alliance).exists():
        messages.error(request, _('Alliance contacts for your alliance are already being tracked.'))
        return redirect('aa_contacts:dashboard')

    AllianceToken.objects.create(alliance=alliance, token=token)
    update_alliance_contacts.delay(alliance.alliance_id)

    messages.success(request, _('Alliance contacts are now being tracked.'))
    return redirect('aa_contacts:dashboard')


@login_required
@permission_required('aa_contacts.manage_corporation_contacts')
@token_required(scopes=['esi-corporations.read_contacts.v1'])
def add_corporation_token(request, token: Token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)

    try:
        corporation = char.corporation
    except EveCorporationInfo.DoesNotExist:
        corporation = EveCorporationInfo.objects.create_corporation(char.corporation_id)

    if CorporationToken.objects.filter(corporation=corporation).exists():
        messages.error(request, _('Corporation contacts for your corporation are already being tracked.'))
        return redirect('aa_contacts:dashboard')

    CorporationToken.objects.create(corporation=corporation, token=token)
    update_corporation_contacts.delay(corporation.corporation_id)

    messages.success(request, _('Corporation contacts are now being tracked.'))
    return redirect('aa_contacts:dashboard')


@login_required
@permission_required('aa_contacts.manage_alliance_contacts')
def update_alliance(request, alliance_pk: int):
    try:
        token = AllianceToken.visible_for(request.user).select_related('alliance').get(alliance_id=alliance_pk)
    except AllianceToken.DoesNotExist:
        messages.error(request, _('You do not have the permissions for viewing this alliance contacts.'))
        return redirect('aa_contacts:dashboard')

    update_alliance_contacts.delay(token.alliance.alliance_id)

    messages.success(request, _('Alliance contacts are being updated.'))
    return redirect('aa_contacts:alliance_contacts', alliance_pk)


@login_required
@permission_required('aa_contacts.manage_corporation_contacts')
def update_corporation(request, corporation_pk: int):
    try:
        token = CorporationToken.visible_for(request.user).select_related('corporation').get(corporation_id=corporation_pk)
    except CorporationToken.DoesNotExist:
        messages.error(request, _('You do not have the permissions for viewing this corporation contacts.'))
        return redirect('aa_contacts:dashboard')

    update_corporation_contacts.delay(token.corporation.corporation_id)

    messages.success(request, _('Corporation contacts are being updated.'))
    return redirect('aa_contacts:corporation_contacts', corporation_pk)


@login_required
@permission_required(['aa_contacts.manage_alliance_contacts', 'aa_contacts.view_alliance_notes'])
def update_alliance_contact(request, contact_pk: int):
    contact = get_object_or_404(AllianceContact, pk=contact_pk)

    if request.method == 'POST':
        form = AllianceContactForm(request.POST, instance=contact)
        if form.is_valid():
            form.save()
            messages.success(request, _('%(contact)s contact updated successfully') % {'contact': contact.contact_name})
            return redirect('aa_contacts:alliance_contacts', contact.alliance_id)
    else:
        form = AllianceContactForm(instance=contact)

    context = {
        'form': form,
        'contact': contact,
    }

    return render(request, 'aa_contacts/edit_contact.html', context=context)


@login_required
@permission_required(['aa_contacts.manage_corporation_contacts', 'aa_contacts.view_corporation_notes'])
def update_corporation_contact(request, contact_pk: int):
    contact = get_object_or_404(CorporationContact, pk=contact_pk)

    if request.method == 'POST':
        form = CorporationContactForm(request.POST, instance=contact)
        if form.is_valid():
            form.save()
            messages.success(request, _('%(contact)s contact updated successfully') % {'contact': contact.contact_name})
            return redirect('aa_contacts:corporation_contacts', contact.corporation_id)
    else:
        form = CorporationContactForm(instance=contact)

    context = {
        'form': form,
        'contact': contact,
    }

    return render(request, 'aa_contacts/edit_contact.html', context=context)



from ninja import Router

from .tokens import router as token_router
from .contacts import router as contacts_router

router = Router()

router.add_router("/tokens", token_router, tags=["tokens"])
router.add_router("{int:corporation_id}/contacts", contacts_router, tags=["contacts"])



from django.test import TestCase

from aa_contacts.forms import AllianceContactForm, CorporationContactForm


class TestForms(TestCase):

    def test_alliance(self):
        form = AllianceContactForm()
        for field, data in form.fields.items():
            if field != 'notes':
                self.assertTrue(data.disabled)
            else:
                self.assertFalse(data.disabled)

    def test_corporation(self):
        form = CorporationContactForm()
        for field, data in form.fields.items():
            if field != 'notes':
                self.assertTrue(data.disabled)
            else:
                self.assertFalse(data.disabled)



from ninja import Router
from django.contrib.auth.models import User

from .schema import UserPermissionsSchema

router = Router(tags=["permissions"])


@router.get("/me", response=UserPermissionsSchema)
def get_me(request):
    user: User = request.user
    return {
        "can_manage_alliance_contacts": user.has_perm("aa_contacts.manage_alliance_contacts"),
        "can_manage_corporation_contacts": user.has_perm("aa_contacts.manage_corporation_contacts"),
    }



from esi.clients import EsiClientProvider

from . import __version__ as app_version, __app_name_ua__ as app_name_ua, __github_url__ as github_url

esi = EsiClientProvider(
    ua_appname=app_name_ua,
    ua_version=app_version,
    ua_url=github_url,
)



from datetime import datetime
from typing import Optional
from ninja import Schema, ModelSchema

from django.contrib.auth.models import User

from allianceauth.eveonline.models import EveCorporationInfo, EveAllianceInfo

from aa_contacts.models import Contact, AllianceContact, CorporationContact

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class UserPermissionsSchema(Schema):
    can_manage_alliance_contacts: bool
    can_manage_corporation_contacts: bool


class EveAllianceSchema(ModelSchema):
    logo_url: str

    class Meta:
        model = EveAllianceInfo
        fields = ["alliance_id", "alliance_name"]

    @staticmethod
    def resolve_logo_url(obj: EveAllianceInfo) -> str:
        return obj.logo_url_32.split('?')[0]


class EveCorporationSchema(ModelSchema):
    alliance: Optional[EveAllianceSchema] = None
    logo_url: str

    class Meta:
        model = EveCorporationInfo
        fields = ["corporation_id", "corporation_name"]

    @staticmethod
    def resolve_logo_url(obj: EveCorporationInfo) -> str:
        return obj.logo_url_32.split('?')[0]


class TokenSchema(Schema):
    last_update: datetime


class AllianceTokenSchema(TokenSchema):
    alliance: EveAllianceSchema


class CorporationTokenSchema(TokenSchema):
    corporation: EveCorporationSchema


class ContactLabelSchema(Schema):
    label_name: str


class ContactSchema(Schema):
    id: int
    contact_id: int
    contact_type: Contact.ContactTypeOptions
    contact_logo_url: str
    contact_name: str
    standing: float
    notes: Optional[str] = None
    can_edit_notes: bool
    labels: list[ContactLabelSchema] = []

    @staticmethod
    def resolve_notes(obj: Contact, context) -> Optional[str]:
        user: User = context['request'].user
        if obj.can_view_notes(user):
            return obj.notes

    @staticmethod
    def resolve_contact_logo_url(obj: Contact) -> str:
        return obj.image_src.split('?')[0]

    @staticmethod
    def resolve_can_edit_notes(obj: Contact, context) -> bool:
        user: User = context['request'].user
        return obj.can_edit_notes(user)


class UpdateContactSchema(Schema):
    notes: str
